﻿EnableExplicit

Global min_zoom = 0
Global max_zoom = 20

Define.l Event
Define.i zoom = 6
Define.d lon_deg = 17.47288403384952, lat_deg = 49.070453838179574


Enumeration
  #winMain
  #fntMain
EndEnumeration

;Gadgets
Enumeration
  #imgMap
  #txtLabelLon
  #txtLabelLat
  #editTextLon
  #editTextLat 
  #txtLabelTileServer
  #cmbURLTileServer
  #txtLabelDataSources
  #chkShowPrecip
  #chkShowTiles
  #txtLabelZoom
  #btnZoomPlus
  #btnZoomMinus
  #stbStatus
EndEnumeration

;Variables
Enumeration
  #G_TYPE_STRING = 64
EndEnumeration

CompilerSelect #PB_Compiler_OS
  CompilerCase #PB_OS_Windows
    Import "gdi32.lib"
      CreateFontIndirectW(*hFont)
    EndImport
  CompilerCase #PB_OS_MacOS
    ; some Mac OS X specific code
  CompilerCase #PB_OS_Linux
    ImportC ""
      g_object_get_property(*widget.GtkWidget, property.p-utf8, *gval)
    EndImport
CompilerEndSelect

ImportC ""
  time(*tloc = #Null)
EndImport

Macro floor(x)
  Round(x, #PB_Round_Down)
EndMacro

Procedure.s GetSystemFont()
  CompilerSelect #PB_Compiler_OS
    CompilerCase #PB_OS_Windows
      Protected phkResult, lpcbData, *lplf.LOGFONT, hLogFont.s
      If RegOpenKeyEx_(#HKEY_CURRENT_USER, "Control Panel\Desktop\WindowMetrics", 0, #KEY_READ, @phkResult) = #ERROR_SUCCESS
        If RegQueryValueEx_(phkResult, "MessageFont", #Null, 0, 0, @lpcbData) = #ERROR_SUCCESS
          *lplf = AllocateMemory(lpcbData)
          If *lplf
            If RegQueryValueEx_(phkResult, "MessageFont", #Null, 0, *lplf, @lpcbData) = #ERROR_SUCCESS
              hLogFont = CreateFontIndirect_(*lplf)
            EndIf
            FreeMemory(*lplf)
          EndIf
        EndIf
        RegCloseKey_(phkResult)
      EndIf
      ProcedureReturn hLogFont
    CompilerCase #PB_OS_MacOS
      ; some Mac OS X specific code
    CompilerCase #PB_OS_Linux
      Protected   gVal.GValue
      Protected.i I
      Protected.s StdFnt, FontName
      g_value_init_(@gval, #G_TYPE_STRING)
      g_object_get_property(gtk_settings_get_default_(), "gtk-font-name", @gval)
      StdFnt= PeekS(g_value_get_string_(@gval), -1, #PB_UTF8)
      g_value_unset_(@gval)
      If Val(StringField(ReverseString(StdFnt), 1, " "))
        For I= 1 To CountString(StdFnt, " ")
          FontName+ StringField(StdFnt, I, " ")
        Next I
      Else
        FontName= StdFnt
      EndIf
      ProcedureReturn FontName
  CompilerEndSelect
EndProcedure

Procedure.I FileExists(Filename.s)
  CompilerSelect #PB_Compiler_OS
    CompilerCase #PB_OS_Windows
      If GetFileAttributes(FileName) & #FILE_ATTRIBUTE_DIRECTORY = #False
        ProcedureReturn #True
      EndIf
    CompilerCase #PB_OS_Linux
      Protected UTF8Filename.s = Space(StringByteLength(Filename, #PB_UTF8) + 1)
      PokeS(@UTF8Filename, Filename, -1, #PB_UTF8)
      ProcedureReturn g_file_test_(UTF8Filename, #G_FILE_TEST_EXISTS)
    CompilerCase #PB_OS_MacOS
      ProcedureReturn CocoaMessage(0, CocoaMessage(0, 0, "NSFileManager defaultManager"), "fileExistsAtPath:$", @Filename)
  CompilerEndSelect
EndProcedure

;Vypocty pro koordináty z https://wiki.openstreetmap.org/wiki/Slippy_map_tilenames
;Obsah je dostupný pod Creative Commons Attribution-ShareAlike 2.0 license, pokud není uvedeno jinak.

Procedure.i CoordToTileX(lon_deg.d, zoom.d)
  ProcedureReturn floor(((lon_deg + 180) / 360) * Pow(2, zoom))
EndProcedure

Procedure.i CoordToTileY(lat_deg.d, zoom.d)
  ProcedureReturn floor((1 - Log(Tan(Radian(lat_deg)) + 1 / Cos(Radian(lat_deg))) / #PI) /2 * Pow(2, zoom))
EndProcedure

Procedure.i CoordToTilePositionX(lon_deg.d, zoom.d)
  ProcedureReturn Mod(((lon_deg + 180) / 360) * Pow(2, zoom), 1) * 256
EndProcedure

Procedure.i CoordToTilePositionY(lat_deg.d, zoom.d)
  ProcedureReturn Mod((1 - Log(Tan(Radian(lat_deg)) + 1 / Cos(Radian(lat_deg))) / #PI) /2 * Pow(2, zoom), 1) * 256
EndProcedure

Procedure MapCenterStation(tile_server.s, lon_deg.d, lat_deg.d, zoom.i, target.i, show_precip = 1, show_tiles = 0,alpha.i = 255, pointer_alpha.i = 255, precip_alpha = 150,  tile_width.i = 256, tile_height.i = 256)
  DisableWindow(0,1)
  Protected drawing_img = CreateImage(#PB_Any, 1280, 1280)
  Protected *Buffer
  Protected Size.d
  Protected center_mapX.d, center_mapY.d
  Protected Text$
  ;Dummy image
  Protected dummy_img = CreateImage(#PB_Any, tile_width, tile_height)
  If StartVectorDrawing(ImageVectorOutput(dummy_img))
    ScaleCoordinates(DesktopResolutionX(), DesktopResolutionY())
    AddPathBox(0, 0, ImageWidth(dummy_img), ImageHeight(dummy_img))
    VectorSourceColor(RGBA(255,255,255,255))
    FillPath()
    ClosePath()
    BeginVectorLayer(alpha)
    VectorSourceColor(RGBA(0, 0, 0, 255))
    MovePathCursor(0,0)
    AddPathLine(ImageWidth(dummy_img), ImageHeight(dummy_img))
    StrokePath(2)
    MovePathCursor(0,ImageHeight(dummy_img))
    AddPathLine(ImageWidth(dummy_img), 0)
    StrokePath(2)
    VectorSourceColor(RGBA(0,0,155,255))
    VectorFont(FontID(#fntMain), 20)
    Text$ = "No data for this zoom"
    MovePathCursor((tile_width/2) - VectorTextWidth(Text$)/2, (tile_height/2) - VectorTextHeight(Text$)/2)
    DrawVectorText(Text$)
    
    EndVectorLayer()
    StopVectorDrawing()
  EndIf 
  
  ;Drawing image from tiles
  InitNetwork() 
  If StartVectorDrawing(ImageVectorOutput(drawing_img))
    ScaleCoordinates(DesktopResolutionX(), DesktopResolutionY())
    AddPathBox(0, 0, ImageWidth(drawing_img), ImageHeight(drawing_img))
    VectorSourceColor(RGBA(255,255,255,255))
    VectorFont(FontID(#fntMain), 20)
    FillPath()
    ClosePath()
    BeginVectorLayer(alpha)
    Protected i.i, j.i
    Protected centerX = CoordToTileX(lon_deg, zoom)
    Protected centerY = CoordToTileY(lat_deg, zoom)
    Protected startX = centerX -2
    Protected stopX = centerX + 2
    Protected startY = centerY - 2
    Protected stopY = centerY + 2
    Protected placeX = 0
    Protected placeY = 0
    Protected act_time_radar = (time() - Mod(time(), 600))
    For j = startY To stopY
      For i = startX To stopX
        MovePathCursor(placeX,placeY)
        *Buffer = ReceiveHTTPMemory(tile_server + Str(zoom) + "/" + i +"/" + j + ".png")
        If *Buffer
          Size = MemorySize(*Buffer)
          CatchImage(0, *Buffer , Size)
          FreeMemory(*Buffer)
          If IsImage(0) = 0
            DrawVectorImage(ImageID(dummy_img), 255)
          Else
            DrawVectorImage(ImageID(0), 255)
            If show_tiles = 1
              VectorSourceColor(RGBA(0,0,155,255))
              Text$ = Str(i) + "/" + Str(j)
              MovePathCursor((placeX + (tile_width/2) - VectorTextWidth(Text$)/2), (placeY + (tile_height/2) - VectorTextHeight(Text$)/2))
              DrawVectorText(Text$)
            EndIf
          EndIf
        Else
          DrawVectorImage(ImageID(dummy_img), 255)
        EndIf
        placeX = placeX + tile_width
      Next
      placeX = 0
      placeY = placeY + tile_height
    Next
    EndVectorLayer()
    
    ;Drawing radar from tiles
    If show_precip = 1
      startX = centerX -2
      stopX = centerX + 2
      startY = centerY - 2
      stopY = centerY + 2
      placeX = 0
      placeY = 0
      BeginVectorLayer(precip_alpha)
      For j = startY To stopY
        For i = startX To stopX
          MovePathCursor(placeX,placeY)
          *Buffer = ReceiveHTTPMemory("https://tilecache.rainviewer.com/v2/radar/" + Str(act_time_radar) + "/256/"+ Str(zoom) + "/" + Str(i) +"/" + Str(j) + "/2/1_0.png")
          If *Buffer
            Size = MemorySize(*Buffer)
            CatchImage(0, *Buffer , Size)
            FreeMemory(*Buffer)
            If IsImage(0) = 0
              DrawVectorImage(ImageID(dummy_img), 255)
            Else
              DrawVectorImage(ImageID(0), 255)
              If show_tiles = 1
                VectorSourceColor(RGBA(0,0,155,255))
                Text$ = Str(i) + "/" + Str(j)
                MovePathCursor((placeX + (tile_width/2) - VectorTextWidth(Text$)/2), (placeY + (tile_height/2) - VectorTextHeight(Text$)/2))
                DrawVectorText(Text$)
              EndIf
            EndIf
          Else
            DrawVectorImage(ImageID(dummy_img), 255)
          EndIf
          placeX = placeX + tile_width
        Next
        placeX = 0
        placeY = placeY + tile_height
      Next
      EndVectorLayer()
    EndIf
    
    
    ;Draw position of station
    BeginVectorLayer(pointer_alpha)
    center_mapX = CoordToTilePositionX(lon_deg, zoom) + tile_width*2
    center_mapY = CoordToTilePositionY(lat_deg,zoom) + tile_height*2
    AddPathCircle(center_mapX , center_mapY, 5, 0, 360)
    VectorSourceColor(RGBA(255, 0, 0, 255))
    StrokePath(5)
    EndVectorLayer()
    
    ;Draw tiles with numbers
    If show_tiles = 1
      BeginVectorLayer(pointer_alpha)
      VectorSourceColor(RGBA(0,0,155,150))
      For i = 1 To 4
        MovePathCursor(i* 256,0)
        AddPathLine(i*256, ImageHeight(drawing_img))
        DotPath(2, 5, #PB_Path_RoundEnd)
        MovePathCursor(0,i*256)
        AddPathLine(ImageWidth(drawing_img), i*256)
        DotPath(2, 5, #PB_Path_RoundEnd)
      Next
      EndVectorLayer()
    EndIf
    StopVectorDrawing()
  EndIf
  FreeImage(dummy_img)
  Protected copy_img = GrabImage(drawing_img, #PB_Any, center_mapX - (GadgetWidth(target)/2), center_mapY - (GadgetHeight(target)/2), GadgetWidth(target), GadgetHeight(target))
  FreeImage(drawing_img)
  SetGadgetState(target, ImageID(copy_img))
  FreeImage(copy_img)
  DisableWindow(0,0)
EndProcedure

Procedure ShowMap(zoom.i)
  MapCenterStation(GetGadgetText(#cmbURLTileServer), ValF(GetGadgetText(#editTextLon)), ValF(GetGadgetText(#editTextLat)), zoom, #imgMap, GetGadgetState(#chkShowPrecip), GetGadgetState(#chkShowTiles), 150) 
  StatusBarText(#stbStatus, 0, " Zoom: " + Str(zoom) + " ", #PB_StatusBar_Center)
  StatusBarText(#stbStatus, 1, " Center tile: " + Str(CoordToTileX(ValF(GetGadgetText(#editTextLon)), zoom)) + "/" + Str(CoordToTileX(ValF(GetGadgetText(#editTextLat)), zoom)), #PB_StatusBar_Center)
EndProcedure

UsePNGImageDecoder()

If OpenWindow(#winMain, 0, 0, 800, 570, "OpenStreetMap with precipitation layer", #PB_Window_SystemMenu | #PB_Window_ScreenCentered)
  LoadFont(#fntMain, GetSystemFont(),20)
  ImageGadget(#imgMap,0,0,800,400,0, #PB_Image_Border)
  TextGadget(#txtLabelLon, 10, 420, 100, 20, "Lon:")
  TextGadget(#txtLabelLat, 10, 460, 100, 20, "Lat:")
  StringGadget(#editTextLon, 50, 410, 150,20,StrD(lon_deg)) 
  StringGadget(#editTextLat, 50, 450, 150,20,StrD(lat_deg))  
  TextGadget(#txtLabelTileServer, 220, 420, 100, 20, "Tile server:")
  ComboBoxGadget(#cmbURLTileServer, 310, 410, 150, 25)
  AddGadgetItem(#cmbURLTileServer,-1,"http://a.tile.stamen.com/toner/")
  AddGadgetItem(#cmbURLTileServer,-1,"https://a.tile.openstreetmap.org/")
  AddGadgetItem(#cmbURLTileServer,-1,"http://b.tile.stamen.com/toner/")
  AddGadgetItem(#cmbURLTileServer,-1,"https://b.tile.openstreetmap.org/")
  AddGadgetItem(#cmbURLTileServer,-1,"http://c.tile.stamen.com/toner/")
  AddGadgetItem(#cmbURLTileServer,-1,"https://c.tile.openstreetmap.org/")
  SetGadgetState(#cmbURLTileServer, 0)
  TextGadget(#txtLabelDataSources,5,500,800,50,"Map tiles by Stamen Design under CC BY 3.0. Data by OpenStreetMap under CC BY SA. Weather Data by RainViewer at https://rainviewer.com.")
  SetGadgetColor(#txtLabelDataSources,  #PB_Gadget_FrontColor, RGB(0,120,0))
  CheckBoxGadget(#chkShowPrecip, 310, 450, 100,20, "Show precipitation") 
  SetGadgetState(#chkShowPrecip, #PB_Checkbox_Checked)
  CheckBoxGadget(#chkShowTiles, 490, 450, 100,20, "Show tiles") 
  SetGadgetState(#chkShowTiles, #PB_Checkbox_Unchecked)
  TextGadget(#txtLabelZoom, 620, 420, 100, 20, "Zoom:")
  ButtonGadget(#btnZoomPlus, 680, 410, 30, 20,"+")
  ButtonGadget(#btnZoomMinus, 740, 410, 30, 20,"-")
  CreateStatusBar(#stbStatus, WindowID(0))
  AddStatusBarField(100)
  AddStatusBarField(200)
  
  ShowMap(zoom) 
  
  Repeat
    Event = WaitWindowEvent()
    Select Event  
      Case #PB_Event_Gadget
        Select EventGadget()
          Case #btnZoomPlus 
            If zoom = max_zoom : Else : zoom = zoom + 1 : EndIf
            ShowMap(zoom)          
          Case #btnZoomMinus 
            If zoom = min_zoom : Else : zoom = zoom - 1 : EndIf
            ShowMap(zoom)          
          Case #cmbURLTileServer 
            Select EventType()
              Case #PB_EventType_Change
                ShowMap(zoom)            
            EndSelect
          Case #chkShowTiles 
            ShowMap(zoom)  
          Case #chkShowPrecip
            ShowMap(zoom) 
        EndSelect
    EndSelect
  Until Event = #PB_Event_CloseWindow
EndIf
; IDE Options = PureBasic 5.72 (Linux - x64)
; CursorPosition = 294
; FirstLine = 265
; Folding = --
; EnableXP
; Executable = MapsFromServerCenterStation